#!/bin/sh
#   Copyright (c) 2010 digitalSTROM.org, Zurich, Switzerland
#
#   Author: Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
#
#   This file is part of digitalSTROM Server.
#
#   netinfo.sh is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   netinfo.sh is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this file. If not, see <http://www.gnu.org/licenses/>.

# source common functions
. ./errlib.sh
. ./jsonlib.sh
. ./netlib.sh

HOSTNAME_FILE="/etc/hostname"
DSS_STATE_FILE="/etc/runit/dss/supervise/stat"
MT_STATE_FILE="/etc/runit/mediatomb/supervise/stat"
MT_CONFIG_FILE="/etc/mediatomb/config.xml"
UPNP_BEACON_CONFIG_FILE="/etc/upnpbeacon.cfg"
UPNP_BEACON_SCRIPT="/etc/init.d/upnpbeacon"

set_hostname() {
    echo "$1" > "$HOSTNAME_FILE"
    if [[ "$?" -ne "0" ]]; then
        json_error $ERR_FWRITE "Could not write to hostname file!"
        return $ERR_FWRITE
    fi    

    hostname -F "$HOSTNAME_FILE" 1>&2
    if [[ "$?" -ne "0" ]]; then
        json_error $ERR_HOSTNAME "Could not set hostname!"
        return $ERR_HOSTNAME
    fi

    SET_HOSTNAME=`hostname`
    if [[ "$SET_HOSTNAME" != "$1" ]]; then
        json_error $ERR_HOSTNAME "Error setting new hostname!"
        echo "dSS11" > "$HOSTNAME_FILE"
        return $ERR_HOSTNAME
    fi

    # restart avahi to announce the new hostname
    /etc/init.d/avahi-daemon restart > /dev/null 2>&1

    RET=$(restart_net_with_redirect)
    RETCODE="$?"
    if [[ "$RETCODE" -ne "$ERR_OK" ]]; then
        json_error $RETCODE "Error restarting network!"
        return $RETCODE
    fi

    SURVIVE=`echo $RET | cut -d ":" -f 2`
    if [[ "$SURVIVE" -eq "1" ]]; then
        redirect_and_quit $RET
    fi
    
    return $ERR_OK
}

set_upnp_friendly_name() {
    if [[ -f "$MT_CONFIG_FILE" ]]; then
        OLD_NAME=$(cat "$MT_CONFIG_FILE" |sed -n -e 's/.*<name>\(.*\)<\/name>.*/\1/p')
        sed -i s:"<name>$OLD_NAME</name>":"<name>$1</name>":g "$MT_CONFIG_FILE"
        
        NEW_NAME=$(cat "$MT_CONFIG_FILE" |sed -n -e 's/.*<name>\(.*\)<\/name>.*/\1/p')

        if [[ "$NEW_NAME" != "$1" ]]; then
            json_error $ERR_UPNPAME "Could not set UPnP friendly name!"
            return $ERR_UPNPNAME
        fi

        MT_STATE=
        if [[ -f "$MT_STATE_FILE" ]]; then
            MT_STATE=`cat "$MT_STATE_FILE"`
            if [[ "$DSS_STATE" == "run" ]]; then
                sv -w 5 down /etc/runit/mediatomb 1>&2
                sleep 1
                sv -w 15 up /etc/runit/mediatomb 1>&2
            fi
        fi

    fi

    if [[ -f "$UPNP_BEACON_CONFIG_FILE" ]]; then
        grep -v "upnp-name" "$UPNP_BEACON_CONFIG_FILE" >/tmp/upnp.rename
        echo "upnp-name = \"$1\";" >> /tmp/upnp.rename
        mv /tmp/upnp.rename "$UPNP_BEACON_CONFIG_FILE"
        if [[ -x "$UPNP_BEACON_SCRIPT" ]]; then
            $UPNP_BEACON_SCRIPT restart 1>&2
        fi
    fi
    return 0
}

echo "Content-Type: application/json"
echo ""


HOSTNAME=""

if [[ -n "$1" ]]; then 
    echo "$1" | grep "[^0-9a-zA-Z-]" > /dev/null 2>&1
    if [ "$?" -eq "0" ]; then
        json_error $ERR_HOSTNAME "Invalid hostname specified!"
        exit 1
    fi
    HOSTNAME="$1"
else
    json_error $ERR_HOSTNAME "Invalid hostname specified!"
    exit 1
fi

set_upnp_friendly_name "$HOSTNAME"
RETCODE="$?"
if [[ "$RETCODE" -ne "$ERR_OK" ]]; then
    exit $RETCODE
fi

set_hostname "$HOSTNAME"
RETCODE="$?"
if [[ "$RETCODE" -ne "$ERR_OK" ]]; then
    exit $RETCODE
fi

json_ok
